From 7bbc85b177cd22dd6eefdf9162a5f1bcee9e78a4 Mon Sep 17 00:00:00 2001
From: Jean-Baptiste Quenot <jbq@nomao.com>
Date: Fri, 25 Feb 2011 12:20:26 +0100
Subject: [PATCH] Improve hstore support

* Discover hstore type in any namespace, not just 'public'
* Allow to register hstore on a RealDictConnection
---
 lib/extras.py |   18 ++++++++++++------
 1 files changed, 12 insertions(+), 6 deletions(-)

diff --git a/lib/extras.py b/lib/extras.py
index 09142ed..dee8d75 100644
--- a/lib/extras.py
+++ b/lib/extras.py
@@ -661,7 +661,7 @@ class HstoreAdapter(object):
     parse_unicode = classmethod(parse_unicode)
 
     @classmethod
-    def get_oids(self, conn_or_curs):
+    def get_oid(self, conn_or_curs):
         """Return the oid of the hstore and hstore[] types.
 
         Return None if hstore is not available.
@@ -684,7 +684,7 @@ class HstoreAdapter(object):
 SELECT t.oid, %s
 FROM pg_type t JOIN pg_namespace ns
     ON typnamespace = ns.oid
-WHERE typname = 'hstore' and nspname = 'public';
+WHERE typname = 'hstore';
 """ % typarray)
         oids = curs.fetchone()
 
@@ -693,7 +693,13 @@ WHERE typname = 'hstore' and nspname = 'public';
         and conn.isolation_level != _ext.ISOLATION_LEVEL_AUTOCOMMIT):
             conn.rollback()
 
-        return oids
+        if oids is None:
+            return None
+
+        if isinstance(oids, dict):
+            return oids['oid']
+
+        return oids[0]
 
 def register_hstore(conn_or_curs, globally=False, unicode=False):
     """Register adapter and typecaster for `dict`\-\ |hstore| conversions.
@@ -714,8 +720,8 @@ def register_hstore(conn_or_curs, globally=False, unicode=False):
     (executing the ``hstore.sql`` script in your ``contrib`` directory).
     Raise `~psycopg2.ProgrammingError` if the type is not found.
     """
-    oids = HstoreAdapter.get_oids(conn_or_curs)
-    if oids is None:
+    oid = HstoreAdapter.get_oid(conn_or_curs)
+    if oid is None:
         raise psycopg2.ProgrammingError(
             "hstore type not found in the database. "
             "please install it from your 'contrib/hstore.sql' file")
@@ -726,7 +732,7 @@ def register_hstore(conn_or_curs, globally=False, unicode=False):
     else:
         cast = HstoreAdapter.parse
 
-    HSTORE = _ext.new_type((oids[0],), "HSTORE", cast)
+    HSTORE = _ext.new_type((oid,), "HSTORE", cast)
     _ext.register_type(HSTORE, not globally and conn_or_curs or None)
     _ext.register_adapter(dict, HstoreAdapter)
 
-- 
1.6.3.3

